////////////////////////////////////////////////////////////////////////
// schedule.cpp
// generated with ecppc
//

#include <tnt/ecpp.h>
#include <tnt/httprequest.h>
#include <tnt/httpreply.h>
#include <tnt/httpheader.h>
#include <tnt/http.h>
#include <tnt/data.h>
#include <tnt/componentfactory.h>
#include <cxxtools/log.h>
#include <stdexcept>

log_define("component.schedule")

// <%pre>
#line 1 "schedule.ecpp"


#include <livefeatures.h>
#include <setup.h>
#include <tools.h>
#include <epg_events.h>
#include <recman.h>


#include <vdr/config.h>
using namespace vdrlive;

bool searchNameDesc(RecordingsItemPtr &RecItem, std::list<RecordingsItemPtr> &RecItems, const std::string &Name, const std::string &ShortText, const std::string &Description, long Duration) {
  if(RecItems.begin() == RecItems.end() ) return false;  // there are no recordings

// find all recordings with equal name
  RecordingsItemPtr dummy (new RecordingsItemDummy(Name, ShortText, Description, Duration));
  std::list<RecordingsItemPtr>::iterator recIterLowName = std::lower_bound (RecItems.begin(), RecItems.end(), dummy, RecordingsItemPtrCompare::ByAscendingName);
  std::list<RecordingsItemPtr>::iterator recIterUpName  = std::upper_bound (recIterLowName  , RecItems.end(), dummy, RecordingsItemPtrCompare::ByAscendingName);
  if ( recIterLowName == recIterUpName ) return false; // there is no recording with this name


// find all recordings with matching short text / description
  std::list<RecordingsItemPtr>::iterator recIterLow = std::lower_bound (recIterLowName, recIterUpName, dummy, RecordingsItemPtrCompare::ByAscendingNameDesc);
  std::list<RecordingsItemPtr>::iterator recIterUp = std::upper_bound (recIterLow, recIterUpName, dummy, RecordingsItemPtrCompare::ByAscendingNameDesc);

  if(recIterLow != recIterUp) {   // exact match found
    if (RecordingsItemPtrCompare::FindBestMatch(RecItem, recIterLow, recIterUp, dummy) > 0) return true;
    RecItem = *recIterLow;
    return true;
  }
// no exact match found, get recording with most matching characters
  int numEqualCharsLow = 0;
  int numEqualCharsUp  = 0;

  if(recIterLow != recIterLowName) { 
    --recIterLow;
    RecordingsItemPtrCompare::Compare2(numEqualCharsLow, *recIterLow, dummy);
  }
  if(recIterUp != recIterUpName)
    RecordingsItemPtrCompare::Compare2(numEqualCharsUp , *recIterUp , dummy);

  if ( numEqualCharsLow > numEqualCharsUp ) {
    if( numEqualCharsLow > 5 ) { RecItem = *recIterLow; return true; }
  } else {
    if( numEqualCharsUp  > 5 ) { RecItem = *recIterUp;  return true; }
  }


// no exact match found, get best match
  int num_match_rec = RecordingsItemPtrCompare::FindBestMatch(RecItem, recIterLowName, recIterUpName, dummy);
  if(num_match_rec == 0 || num_match_rec > 5) return false; // no matching lenght or series (too many matching length)
  return true;
}



#line 1 "page_init.eh"

// do not add to Makefile
// and do not write a ecpp comment into this file. It must produce no
// html output not even an empty line.
#include <tnt/savepoint.h>
#include <exception.h>

#line 2 "page_exit.eh"

#include <exception.h>

// </%pre>

namespace
{
class _component_ : public tnt::EcppComponent
{
    _component_& main()  { return *this; }

  protected:
    ~_component_();

  public:
    _component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);

    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
    class channel_selection_type : public tnt::EcppSubComponent
    {
        _component_& mainComp;
        _component_& main()  { return mainComp; }

      public:
        channel_selection_type(_component_& m, const std::string& name)
          : EcppSubComponent(m, name),
            mainComp(m)
          { }
        unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
    };

    class rec_item_file_type : public tnt::EcppSubComponent
    {
        _component_& mainComp;
        _component_& main()  { return mainComp; }

      public:
        rec_item_file_type(_component_& m, const std::string& name)
          : EcppSubComponent(m, name),
            mainComp(m)
          { }
        unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);
    };

    channel_selection_type channel_selection;
    rec_item_file_type rec_item_file;
};

static tnt::EcppComponentFactoryImpl<_component_> Factory("schedule");

static const char* rawData = "\134\001\000\000]\001\000\000^\001\000\000\202\001\000\000\215\001\000\000\220\001\000"
  "\000\326\001\000\000\331\001\000\000\361\001\000\000\364\001\000\000\366\001\000\0000\002\000\000i\002\000\000\265"
  "\002\000\000\327\002\000\000\377\002\000\000\001\003\000\000\036\003\000\000 \003\000\000H\003\000\000T\003\000\000]"
  "\003\000\000h\003\000\000\236\003\000\000\273\003\000\000\275\003\000\000\336\003\000\000\370\003\000\000\373\003\000"
  "\000\001\004\000\000\035\004\000\0002\004\000\000T\004\000\000i\004\000\000\212\004\000\000\220\004\000\000\262\004"
  "\000\000\270\004\000\000\277\004\000\000\300\004\000\000\314\004\000\000\346\004\000\000\347\004\000\000\350\004\000"
  "\000E\005\000\000_\005\000\000t\005\000\000z\005\000\000\200\005\000\000\206\005\000\000\324\005\000\000\034\006\000"
  "\000*\006\000\000+\006\000\000-\006\000\000\207\006\000\000\212\006\000\000\244\006\000\000\261\006\000\000\300\006"
  "\000\000\313\006\000\000%\007\000\000(\007\000\000/\007\000\000F\007\000\000`\007\000\000a\007\000\000b\007\000\000h"
  "\007\000\000\224\007\000\000\227\007\000\000\360\007\000\000\010\010\000\000f\010\000\000~\010\000\000\246\010\000"
  "\000\307\010\000\000\334\010\000\000\375\010\000\000\003\t\000\000\026\t\000\000)\t\000\000/\t\000\0000\t\000\0001\t"
  "\000\0002\t\000\0003\t\000\000\n\n\n<html>\n\t<head>\n\t\t<title>VDR Live - </title>\n\t\t\n\t\t\n\t</head>\n\t<body "
  "onload=\"adjustHeader()\" onresize=\"adjustHeader()\">\n\t\t\n\t\t\n\t\t<div class=\"inhalt\">\n\t\t\t.\n\t\t<table c"
  "lass=\"listing\" cellspacing=\"0\" cellpadding=\"0\">\n\t\t\t\t<tr class=\"spacer\">\n\t\t\t\t\t<td colspan=\"5\"/>\n"
  "\t\t\t\t</tr>\n\t\t\t\t<tr class=\"head\">\n\t\t\t\t\t<td colspan=\"5\"><div class=\"boxheader\"><div><div></div></di"
  "v></div></td>\n\t\t\t\t</tr>\n\t\t\t\t<tr>\n\t\t\t\t\t<td class=\"action leftcol \"></td>\n\t\t\t\t\t<td class=\"acti"
  "on \"><a href=\"searchresults.html\?searchplain=\"><img src=\"\" alt=\"\" ></img></a><img src=\"img/transparent.png\""
  " width=\"16\" height=\"16\"></td>\n\t\t\t\t\t<td class=\"action \"></td>\n\t\t\t\t\t<td class=\"topaligned \"><div cl"
  "ass=\"withmargin\"> - <br /></div></td>\n\t\t\t\t\t<td class=\" topaligned rightcol \"><div class=\"more withmargin\""
  "><a ><span class=\"title\"></span><br /><span class=\"short\">&nbsp;</span></a></div></td>\n\n\t\t\t\t</tr>\n<tr>\n "
  "\n</tr>\n\n\t\t\t</table>\n\t\t</div>\n\t</body>\n</html>\n\n\n<form action=\"schedule.html\" method=\"get\" id=\"cha"
  "nnels\">\n   <span>\n     <label for=\"channel\">:&nbsp;<span class=\"bold\"></span></label>\n     \n     \n     \n  "
  "   \n   </span>\n</form>\n<span class=\"sep\">|</span>\n<a href=\"schedule.html\?channel=\">\302\253 Prev</a>\n<span "
  "class=\"sep\">|</span>\n<a href=\"schedule.html\?channel=\">Next \302\273</a>\n\n\n\n<td colspan=\"2\" style=\"word-w"
  "rap: normal;min-width: 10px;max-width: 70px;\" class=\"leftcol \">   </td>\n<td class=\"action \"> <img src=\"\" alt="
  "\"on_dvd\"  /> </td>\n\n<td colspan=\"3\" style=\"word-wrap: normal;min-width: 10px;max-width: 70px;\" class=\"leftco"
  "l \">  </td>\n\n<td class=\"topaligned \"><div class=\"withmargin\">  <br /></div></td>\n\n<td class=\"topaligned rig"
  "htcol \">\n<table><tr><td class=\"noborder\">\n                <div class=\"recording_errors\"><img src=\"\" width = "
  "\"16px\" alt=\"\"  /> </div>\n</td>\n<td class=\"noborder\">\n                <div class=\"recording_sd_hd\"><img src"
  "=\"\" width = \"25px\" alt=\"\"  /> </div>\n</td>\n\n<td class=\"noborder\">\n\n<div class=\"more withmargin\"><a ><s"
  "pan class=\"title\"></span><br /><span class=\"short\">&nbsp;</span></a></div>\n\n</td></tr></table>\n</td>\n\n\n\n\n";

// <%shared>
// </%shared>

// <%config>
// </%config>

_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)
  : EcppComponent(ci, um, cl),
    channel_selection(*this, "channel_selection"),
    rec_item_file(*this, "rec_item_file")
{
  // <%init>
  // </%init>
}

_component_::~_component_()
{
  // <%cleanup>
  // </%cleanup>
}

template <typename T>
inline void _tnt_ignore_unused(const T&) { }

unsigned _component_::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("schedule " << qparam.getUrl());

  tnt::DataChunks _tntChunks(rawData);

  // <%args>
int channel = qparam.get< int >("channel", ( -1));
  // </%args>

#line 62 "schedule.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_SHARED_VAR(bool, logged_in, (false));   // <%session> bool logged_in(false)
  _tnt_ignore_unused<logged_in_type>(logged_in);
#line 67 "schedule.ecpp"
  typedef cChannel* Channel_type;
  TNT_REQUEST_SHARED_VAR(cChannel*, Channel, ());   // <%request> cChannel* Channel
  _tnt_ignore_unused<Channel_type>(Channel);
#line 9 "page_init.eh"
  typedef std::string pageTitle_type;
  TNT_REQUEST_SHARED_VAR(std::string, pageTitle, ());   // <%request> std::string pageTitle
  _tnt_ignore_unused<pageTitle_type>(pageTitle);
  // <%cpp>
  reply.out() << _tntChunks[0]; // \n
  reply.out() << _tntChunks[1]; // \n
  // <%include> page_init.eh
#line 11 "page_init.eh"

try {
	reply.setHeader("Expires", "Mon, 26 Jul 1997 05:00:00 GMT");
	tnt::Savepoint spoint( reply );

  // </%include>
#line 70 "schedule.ecpp"

if (!logged_in && LiveSetup().UseAuth()) return reply.redirect("login.html");

#line 73 "schedule.ecpp"

	pageTitle = trVDR("Schedule");

std::vector<std::string> path;
std::list<RecordingsItemPtr> recItems;
RecordingsTreePtr recordingsTree(LiveRecordingsManager()->GetRecordingsTree());

addAllRecordings(recItems, recordingsTree, path);
recItems.sort(RecordingsItemPtrCompare::ByAscendingNameDesc);


#if VDRVERSNUM < 20301
	cSchedulesLock schedulesLock;
	cSchedules const* schedules = cSchedules::Schedules( schedulesLock );

	ReadLock channelsLock( Channels );
	if ( !channelsLock )
		throw HtmlError( tr("Couldn't aquire access to channels, please try again later.") );
#endif

	// cChannel* Channel; (see %request above)
	if ( channel > 0 ) {
#if VDRVERSNUM >= 20301
		LOCK_CHANNELS_READ;
		Channel = (cChannel *)Channels->GetByNumber( channel );
#else
		Channel = Channels.GetByNumber( channel );
#endif
	}
	else {
#if VDRVERSNUM >= 20301
		LOCK_CHANNELS_READ;
#endif
		if (cDevice::CurrentChannel()) {
#if VDRVERSNUM >= 20301
			Channel = (cChannel *)Channels->GetByNumber(cDevice::CurrentChannel());
#else
			Channel = Channels.GetByNumber(cDevice::CurrentChannel());
#endif
		}
		else {
#if VDRVERSNUM >= 20301
			Channel = (cChannel *)Channels->Get( Channels->GetNextNormal( -1 ) );
#else
			Channel = Channels.Get( Channels.GetNextNormal( -1 ) );
#endif
		}
	}
	if ( Channel == 0 )
		throw HtmlError( tr("Couldn't find channel or no channels available. Maybe you mistyped your request?") );

	cSchedule const* Schedule;
#if VDRVERSNUM >= 20301
	{
		LOCK_SCHEDULES_READ;
		Schedule = Schedules->GetSchedule( (const cChannel *)Channel );
	}
#else
	Schedule = schedules->GetSchedule( Channel );
#endif

  // <& pageelems.doc_type ...
  { 
#line 134 "schedule.ecpp"
    tnt::QueryParams _tnt_cq0;
#line 134 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "doc_type"), request, reply, _tnt_cq0);
    // &>
  }
  reply.out() << _tntChunks[2]; // \n<html>\n\t<head>\n\t\t<title>VDR Live - 
#line 137 "schedule.ecpp"
  reply.sout() << ( pageTitle );
  reply.out() << _tntChunks[3]; // </title>\n\t\t
  // <& pageelems.stylesheets ...
  { 
#line 138 "schedule.ecpp"
    tnt::QueryParams _tnt_cq1;
#line 138 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "stylesheets"), request, reply, _tnt_cq1);
    // &>
  }
  reply.out() << _tntChunks[4]; // \n\t\t
  // <& pageelems.ajax_js ...
  { 
#line 139 "schedule.ecpp"
    tnt::QueryParams _tnt_cq2;
#line 139 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "ajax_js"), request, reply, _tnt_cq2);
    // &>
  }
  reply.out() << _tntChunks[5]; // \n\t</head>\n\t<body onload="adjustHeader()" onresize="adjustHeader()">\n\t\t
  // <& pageelems.logo ...
  { 
#line 142 "schedule.ecpp"
    tnt::QueryParams _tnt_cq3;
#line 142 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "logo"), request, reply, _tnt_cq3);
    // &>
  }
  reply.out() << _tntChunks[6]; // \n\t\t
  // <& menu ...
  { 
#line 143 "schedule.ecpp"
    tnt::QueryParams _tnt_cq4;
#line 143 "schedule.ecpp"
    _tnt_cq4.add( "active", ("schedule"));
#line 143 "schedule.ecpp"
    _tnt_cq4.add( "component", ("schedule.channel_selection"));
#line 143 "schedule.ecpp"
    callComp(tnt::Compident(std::string(), "menu"), request, reply, _tnt_cq4);
    // &>
  }
  reply.out() << _tntChunks[7]; // \n\t\t<div class="inhalt">\n
#line 145 "schedule.ecpp"

	if ( Schedule == 0 ) {

  reply.out() << _tntChunks[8]; // \t\t\t
#line 148 "schedule.ecpp"
  reply.sout() << ( tr("No schedules available for this channel") );
  reply.out() << _tntChunks[9]; // .\n
#line 149 "schedule.ecpp"

	}
	else {

  reply.out() << _tntChunks[10]; // \t\t<table class="listing" cellspacing="0" cellpadding="0">\n
#line 154 "schedule.ecpp"

		std::string current_day = "";
		const cEvent* PresentEvent = Schedule->GetPresentEvent();
		time_t now = time(NULL) - ::Setup.EPGLinger * 60;
                RecordingsItemPtr recItem;
                bool recItemFound;
#if VDRVERSNUM >= 20301
		for (const cEvent *Event = (cEvent *)Schedule->Events()->First(); Event; Event = (cEvent *)Schedule->Events()->Next(Event)) {
#else
		for (const cEvent *Event = Schedule->Events()->First(); Event; Event = Schedule->Events()->Next(Event)) {
#endif
			if (Event->EndTime() <= now && Event != PresentEvent)
				continue;

			EpgInfoPtr epgEvent = EpgEvents::CreateEpgInfo(Channel, Event);

			std::string title(epgEvent->Title());
			std::string short_description(epgEvent->ShortDescr());
			std::string description(epgEvent->LongDescr());
			std::string start(epgEvent->StartTime(tr("%I:%M %p")));
			std::string end(epgEvent->EndTime(tr("%I:%M %p")));
                        int durationMinutes = epgEvent->Duration() / 60;
                        std::string duration(durationMinutes < 0 ? "" : FormatDuration(tr("(%d:%02d)"), durationMinutes / 60, durationMinutes % 60));
			std::string day(epgEvent->StartTime(tr("%A, %b %d %Y")));
			std::string epgid = EpgEvents::EncodeDomId(Channel->GetChannelID(), Event->EventID());

			bool truncated = false;
			bool lastEventCurrentDay = false;

                        recItemFound = searchNameDesc(recItem, recItems, title, short_description, description, epgEvent->Duration() );
			{
#if VDRVERSNUM >= 20301
				cEvent* NextEvent = (cEvent *)Schedule->Events()->Next(Event);
#else
				cEvent* NextEvent = Schedule->Events()->Next(Event);
#endif
				if (!NextEvent) {
					lastEventCurrentDay = true;
				}
				else {
					std::string nday(NextEvent->StartTime() ? FormatDateTime(tr("%A, %b %d %Y"), NextEvent->StartTime()) : "");
					lastEventCurrentDay = (day != nday);
				}
			}

			if (current_day != day) {
				if (current_day != "") {

  reply.out() << _tntChunks[11]; // \t\t\t\t<tr class="spacer">\n\t\t\t\t\t<td colspan="5"/>\n\t\t\t\t</tr>\n
#line 205 "schedule.ecpp"

				}

  reply.out() << _tntChunks[12]; // \t\t\t\t<tr class="head">\n\t\t\t\t\t<td colspan="5"><div class="boxheader"><div><div>
#line 209 "schedule.ecpp"
  reply.sout() << ( day );
  reply.out() << _tntChunks[13]; // </div></div></div></td>\n\t\t\t\t</tr>\n
#line 211 "schedule.ecpp"

				current_day = day;
			}

  reply.out() << _tntChunks[14]; // \t\t\t\t<tr>\n\t\t\t\t\t<td class="action leftcol 
#line 216 "schedule.ecpp"
  if ( lastEventCurrentDay && ! recItemFound )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[15]; // ">
  // <& pageelems.event_timer ...
  { 
#line 216 "schedule.ecpp"
    tnt::QueryParams _tnt_cq5;
#line 216 "schedule.ecpp"
    _tnt_cq5.add( "epgid", (epgid));
#line 216 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "event_timer"), request, reply, _tnt_cq5);
    // &>
  }
  reply.out() << _tntChunks[16]; // </td>\n\t\t\t\t\t<td class="action 
#line 217 "schedule.ecpp"
  if ( lastEventCurrentDay && ! recItemFound )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[17]; // ">
#line 217 "schedule.ecpp"
if (LiveFeatures<features::epgsearch>().Recent() ) { 
  reply.out() << _tntChunks[18]; // <a href="searchresults.html\?searchplain=
#line 217 "schedule.ecpp"
  reply.sout() << ( StringUrlEncode(title) );
  reply.out() << _tntChunks[19]; // "><img src="
#line 217 "schedule.ecpp"
  reply.sout() << ( LiveSetup().GetThemedLink("img", "search.png") );
  reply.out() << _tntChunks[20]; // " alt="" 
  // <& tooltip.hint ...
  { 
#line 217 "schedule.ecpp"
    tnt::QueryParams _tnt_cq6;
#line 217 "schedule.ecpp"
    _tnt_cq6.add( "text", (tr("Search for repeats.")));
#line 217 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq6);
    // &>
  }
  reply.out() << _tntChunks[21]; // ></img></a>
#line 217 "schedule.ecpp"
 } else { 
  reply.out() << _tntChunks[22]; // <img src="img/transparent.png" width="16" height="16">
#line 217 "schedule.ecpp"
 } 
  reply.out() << _tntChunks[23]; // </td>\n\t\t\t\t\t<td class="action 
#line 218 "schedule.ecpp"
  if ( lastEventCurrentDay && ! recItemFound )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[24]; // ">
  // <& pageelems.imdb_info_href ...
  { 
#line 218 "schedule.ecpp"
    tnt::QueryParams _tnt_cq7;
#line 218 "schedule.ecpp"
    _tnt_cq7.add( "title", (title));
#line 218 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "imdb_info_href"), request, reply, _tnt_cq7);
    // &>
  }
  reply.out() << _tntChunks[25]; // </td>\n\t\t\t\t\t<td class="topaligned 
#line 219 "schedule.ecpp"
  if ( lastEventCurrentDay && ! recItemFound )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[26]; // "><div class="withmargin">
#line 219 "schedule.ecpp"
  reply.sout() << ( start );
  reply.out() << _tntChunks[27]; //  - 
#line 219 "schedule.ecpp"
  reply.sout() << ( end );
  reply.out() << _tntChunks[28]; // <br />
#line 219 "schedule.ecpp"
  reply.sout() << ( duration );
  reply.out() << _tntChunks[29]; // </div></td>\n\t\t\t\t\t<td class="
#line 220 "schedule.ecpp"
  if ( (Event == PresentEvent) )
    reply.sout() <<  "current" ;
  reply.out() << _tntChunks[30]; //  topaligned rightcol 
#line 220 "schedule.ecpp"
  if ( lastEventCurrentDay && ! recItemFound )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[31]; // "><div class="more withmargin"><a 
  // <& tooltip.hint ...
  { 
#line 220 "schedule.ecpp"
    tnt::QueryParams _tnt_cq8;
#line 220 "schedule.ecpp"
    _tnt_cq8.add( "text", (StringEscapeAndBreak(StringWordTruncate(description, 300, truncated)) + "<br />" + tr("Click to view details.")));
#line 220 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq8);
    // &>
  }
  // <& tooltip.display ...
  { 
#line 220 "schedule.ecpp"
    tnt::QueryParams _tnt_cq9;
#line 220 "schedule.ecpp"
    _tnt_cq9.add( "domId", (epgEvent->Id()));
#line 220 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "display"), request, reply, _tnt_cq9);
    // &>
  }
  reply.out() << _tntChunks[32]; // ><span class="title">
#line 220 "schedule.ecpp"
  reply.sout() << ( title );
  reply.out() << _tntChunks[33]; // </span><br /><span class="short">
#line 220 "schedule.ecpp"
if (short_description.empty()) { 
  reply.out() << _tntChunks[34]; // &nbsp;
#line 220 "schedule.ecpp"
 } 
#line 220 "schedule.ecpp"
  reply.sout() << ( short_description );
  reply.out() << _tntChunks[35]; // </span></a></div></td>\n\n\t\t\t\t</tr>\n
#line 223 "schedule.ecpp"

if(recItemFound) {
                  std::string duration(recItem->Duration() < 0 ? "" : FormatDuration(tr("(%d:%02d)"), recItem->Duration() / 60, recItem->Duration() % 60));
                  std::string shortDescr(recItem->RecInfo()->ShortText() ? recItem->RecInfo()->ShortText() : "");
#if VDRVERSNUM >= 20505
                  int recordingErrors = recItem->RecInfo()->Errors();
#else
                  int recordingErrors = -1;
#endif
                  std::string channelName(recItem->RecInfo()->ChannelName()  ? recItem->RecInfo()->ChannelName() : "");
                  const char *SD_HD_icon = recItem->SD_HD_icon();

                  std::string r_description(recItem->RecInfo()->Description() ? recItem->RecInfo()->Description() : "");
                  std::string hint(tr("Click to view details."));
                  if (!shortDescr.empty()) hint = shortDescr + "<br />" + hint;
                  else if (!r_description.empty()) hint = r_description + "<br />" + hint;
          std::string name(recItem->Name());
          if( *(const char *)recItem->Recording()->Folder() ) {
             name.append(" (");
             name.append( (const char *)recItem->Recording()->Folder() );
             name.append(")");
          }


  reply.out() << _tntChunks[36]; // <tr>\n 
  // <& rec_item_file ...
  { 
#line 248 "schedule.ecpp"
    tnt::QueryParams _tnt_cq10;
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "SD_HD_icon", (SD_HD_icon));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "archived", (RecordingsManager::GetArchiveDescr(recItem->Recording())));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "channelName", (channelName));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "duration", (duration));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "hint", (hint));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "id", (recItem->Id()));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "lastEventCurrentDay", (lastEventCurrentDay));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "level", (recItem->Level()));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "name", (name));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "recordingErrors", (recordingErrors));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "shortDescr", (shortDescr));
#line 248 "schedule.ecpp"
    _tnt_cq10.add( "startTime", (recItem->StartTime()));
#line 248 "schedule.ecpp"
  main().rec_item_file(request, reply, _tnt_cq10);
    // &>
  }
  reply.out() << _tntChunks[37]; // \n</tr>\n
#line 250 "schedule.ecpp"

}

  reply.out() << _tntChunks[38]; // \n
#line 254 "schedule.ecpp"

		}

  reply.out() << _tntChunks[39]; // \t\t\t</table>\n
#line 258 "schedule.ecpp"

	}

  reply.out() << _tntChunks[40]; // \t\t</div>\n\t</body>\n</html>\n
  // <%include> page_exit.eh
  reply.out() << _tntChunks[41]; // \n
#line 5 "page_exit.eh"

	spoint.commit();
} catch ( vdrlive::HtmlError const& ex ) {
	tnt::QueryParams param = qparam;
	param.add( "pageTitle", pageTitle );
	param.add( "errorMessage", ex.what() );
	callComp( "error", request, reply, param );
}

  // </%include>
  reply.out() << _tntChunks[42]; // \n
  reply.out() << _tntChunks[52]; // \n
  reply.out() << _tntChunks[53]; // \n\n
  reply.out() << _tntChunks[82]; // \n
  reply.out() << _tntChunks[83]; // \n
  reply.out() << _tntChunks[84]; // \n
  reply.out() << _tntChunks[85]; // \n
  // <%/cpp>
  return DEFAULT;
}

unsigned _component_::channel_selection_type::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("schedule::channel_selection " << qparam.getUrl());
  tnt::DataChunks _tntChunks(rawData);
#line 62 "schedule.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_SHARED_VAR(bool, logged_in, (false));   // <%session> bool logged_in(false)
  _tnt_ignore_unused<logged_in_type>(logged_in);
#line 67 "schedule.ecpp"
  typedef cChannel* Channel_type;
  TNT_REQUEST_SHARED_VAR(cChannel*, Channel, ());   // <%request> cChannel* Channel
  _tnt_ignore_unused<Channel_type>(Channel);
#line 9 "page_init.eh"
  typedef std::string pageTitle_type;
  TNT_REQUEST_SHARED_VAR(std::string, pageTitle, ());   // <%request> std::string pageTitle
  _tnt_ignore_unused<pageTitle_type>(pageTitle);
  // <%cpp>
  reply.out() << _tntChunks[43]; // <form action="schedule.html" method="get" id="channels">\n   <span>\n     <label for="channel">
#line 269 "schedule.ecpp"
  reply.sout() << ( tr("Channel") );
  reply.out() << _tntChunks[44]; // :&nbsp;<span class="bold">
#line 269 "schedule.ecpp"
  reply.sout() << ( std::to_string(Channel->Number()) );
  reply.out() << _tntChunks[45]; // </span></label>\n     
  // <& channels_widget ...
  { 
#line 270 "schedule.ecpp"
    tnt::QueryParams _tnt_cq11;
#line 270 "schedule.ecpp"
    _tnt_cq11.add( "name", ("channel"));
#line 270 "schedule.ecpp"
    _tnt_cq11.add( "onchange", ("document.forms.channels.submit()"));
#line 270 "schedule.ecpp"
    _tnt_cq11.add( "selected", (Channel ? *Channel->GetChannelID().ToString() : ""));
#line 270 "schedule.ecpp"
    callComp(tnt::Compident(std::string(), "channels_widget"), request, reply, _tnt_cq11);
    // &>
  }
  reply.out() << _tntChunks[46]; // \n     
  // <& pageelems.ajax_action_href ...
  { 
#line 271 "schedule.ecpp"
    tnt::QueryParams _tnt_cq12;
#line 271 "schedule.ecpp"
    _tnt_cq12.add( "action", "switch_channel");
#line 271 "schedule.ecpp"
    _tnt_cq12.add( "alt", "");
#line 271 "schedule.ecpp"
    _tnt_cq12.add( "image", "zap.png");
#line 271 "schedule.ecpp"
    _tnt_cq12.add( "param", (Channel->GetChannelID()));
#line 271 "schedule.ecpp"
    _tnt_cq12.add( "tip", (tr("Switch to this channel.")));
#line 271 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "ajax_action_href"), request, reply, _tnt_cq12);
    // &>
  }
  reply.out() << _tntChunks[47]; // \n     
  // <& pageelems.hls_channel ...
  { 
#line 272 "schedule.ecpp"
    tnt::QueryParams _tnt_cq13;
#line 272 "schedule.ecpp"
    _tnt_cq13.add( "channelId", (Channel->GetChannelID()));
#line 272 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "hls_channel"), request, reply, _tnt_cq13);
    // &>
  }
  reply.out() << _tntChunks[48]; // \n     
  // <& pageelems.m3u_playlist_channel ...
  { 
#line 273 "schedule.ecpp"
    tnt::QueryParams _tnt_cq14;
#line 273 "schedule.ecpp"
    _tnt_cq14.add( "channelId", (Channel->GetChannelID()));
#line 273 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "pageelems", "m3u_playlist_channel"), request, reply, _tnt_cq14);
    // &>
  }
  reply.out() << _tntChunks[49]; // \n   </span>\n</form>\n<span class="sep">|</span>\n<a href="schedule.html\?channel=
#line 277 "schedule.ecpp"
  reply.sout() << ( std::max(0, ((int)Channel->Number() - 1)) );
  reply.out() << _tntChunks[50]; // ">\302\253 Prev</a>\n<span class="sep">|</span>\n<a href="schedule.html\?channel=
#line 279 "schedule.ecpp"
  reply.sout() << ( (Channel->Number() + 1) );
  reply.out() << _tntChunks[51]; // ">Next \302\273</a>\n
  // <%/cpp>
  return DEFAULT;
}

unsigned _component_::rec_item_file_type::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)
{
  log_trace("schedule::rec_item_file " << qparam.getUrl());
  tnt::DataChunks _tntChunks(rawData);
  // <%args>
std::string name = qparam.get< std::string >("name");
int level = qparam.get< int >("level");
std::string id = qparam.get< std::string >("id");
time_t startTime = qparam.get< time_t >("startTime");
std::string duration = qparam.get< std::string >("duration");
std::string hint = qparam.get< std::string >("hint");
std::string shortDescr = qparam.get< std::string >("shortDescr");
int recordingErrors = qparam.get< int >("recordingErrors");
std::string channelName = qparam.get< std::string >("channelName");
std::string SD_HD_icon = qparam.get< std::string >("SD_HD_icon");
std::string archived = qparam.get< std::string >("archived");
bool lastEventCurrentDay = qparam.get< bool >("lastEventCurrentDay");
  // </%args>

#line 62 "schedule.ecpp"
  typedef bool logged_in_type;
  TNT_SESSION_SHARED_VAR(bool, logged_in, (false));   // <%session> bool logged_in(false)
  _tnt_ignore_unused<logged_in_type>(logged_in);
#line 67 "schedule.ecpp"
  typedef cChannel* Channel_type;
  TNT_REQUEST_SHARED_VAR(cChannel*, Channel, ());   // <%request> cChannel* Channel
  _tnt_ignore_unused<Channel_type>(Channel);
#line 9 "page_init.eh"
  typedef std::string pageTitle_type;
  TNT_REQUEST_SHARED_VAR(std::string, pageTitle, ());   // <%request> std::string pageTitle
  _tnt_ignore_unused<pageTitle_type>(pageTitle);
  // <%cpp>
#line 299 "schedule.ecpp"
 if (!archived.empty()) { 
  reply.out() << _tntChunks[54]; // <td colspan="2" style="word-wrap: normal;min-width: 10px;max-width: 70px;" class="leftcol 
#line 300 "schedule.ecpp"
  if ( lastEventCurrentDay )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[55]; // "> 
#line 300 "schedule.ecpp"
  reply.sout() << ( tr("Existing Recording:") );
  reply.out() << _tntChunks[56]; //   </td>\n<td class="action 
#line 301 "schedule.ecpp"
  if ( lastEventCurrentDay )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[57]; // "> <img src="
#line 301 "schedule.ecpp"
  reply.sout() << ( LiveSetup().GetThemedLink("img", "on_dvd.png") );
  reply.out() << _tntChunks[58]; // " alt="on_dvd" 
  // <& tooltip.hint ...
  { 
#line 301 "schedule.ecpp"
    tnt::QueryParams _tnt_cq15;
#line 301 "schedule.ecpp"
    _tnt_cq15.add( "text", (archived));
#line 301 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq15);
    // &>
  }
  reply.out() << _tntChunks[59]; //  /> </td>\n\n
#line 303 "schedule.ecpp"
 } else { 
  reply.out() << _tntChunks[60]; // <td colspan="3" style="word-wrap: normal;min-width: 10px;max-width: 70px;" class="leftcol 
#line 304 "schedule.ecpp"
  if ( lastEventCurrentDay )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[61]; // "> 
#line 304 "schedule.ecpp"
  reply.sout() << ( tr("Existing Recording:") );
  reply.out() << _tntChunks[62]; //  </td>\n
#line 305 "schedule.ecpp"
 } 
  reply.out() << _tntChunks[63]; // \n<td class="topaligned 
#line 307 "schedule.ecpp"
  if ( lastEventCurrentDay )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[64]; // "><div class="withmargin">
#line 307 "schedule.ecpp"
  reply.sout() << ( FormatDateTime(tr("%a,"), startTime) );
  reply.out() << _tntChunks[65]; //  
#line 307 "schedule.ecpp"
  reply.sout() << ( FormatDateTime(tr("%b %d %y"), startTime) );
  reply.out() << _tntChunks[66]; //  
#line 307 "schedule.ecpp"
  reply.sout() << ( FormatDateTime(tr("%I:%M %p"), startTime) );
  reply.out() << _tntChunks[67]; // <br />
#line 307 "schedule.ecpp"
  reply.sout() << ( duration );
  reply.out() << _tntChunks[68]; // </div></td>\n\n<td class="topaligned rightcol 
#line 309 "schedule.ecpp"
  if ( lastEventCurrentDay )
    reply.sout() <<  "bottomrow" ;
  reply.out() << _tntChunks[69]; // ">\n
#line 310 "schedule.ecpp"

#if VDRVERSNUM >= 20505
std::string recording_errors_icon;
std::string recordingErrorsStr;

if (recordingErrors == 0) {
  recording_errors_icon = "NoRecordingErrors.png";
  recordingErrorsStr = tr("No recording errors");
}
if (recordingErrors == -1) {
  recording_errors_icon = "NotCheckedForRecordingErrors.png";
  recordingErrorsStr = tr("Recording errors unknown") ;
}
if (recordingErrors >   0) {
  recording_errors_icon = "RecordingErrors.png";
  recordingErrorsStr = tr("Number of recording errors:");
  recordingErrorsStr += " " + std::to_string(recordingErrors);
}

  reply.out() << _tntChunks[70]; // <table><tr><td class="noborder">\n                <div class="recording_errors"><img src="
#line 330 "schedule.ecpp"
  reply.sout() << ( LiveSetup().GetThemedLink("img", recording_errors_icon) );
  reply.out() << _tntChunks[71]; // " width = "16px" alt="" 
  // <& tooltip.hint ...
  { 
#line 330 "schedule.ecpp"
    tnt::QueryParams _tnt_cq16;
#line 330 "schedule.ecpp"
    _tnt_cq16.add( "text", (recordingErrorsStr));
#line 330 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq16);
    // &>
  }
  reply.out() << _tntChunks[72]; //  /> </div>\n</td>\n<td class="noborder">\n                <div class="recording_sd_hd"><img src="
#line 333 "schedule.ecpp"
  reply.sout() << ( LiveSetup().GetThemedLink("img", SD_HD_icon) );
  reply.out() << _tntChunks[73]; // " width = "25px" alt="" 
  // <& tooltip.hint ...
  { 
#line 333 "schedule.ecpp"
    tnt::QueryParams _tnt_cq17;
#line 333 "schedule.ecpp"
    _tnt_cq17.add( "text", (channelName));
#line 333 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq17);
    // &>
  }
  reply.out() << _tntChunks[74]; //  /> </div>\n</td>\n\n<td class="noborder">\n
#line 337 "schedule.ecpp"

#endif

  reply.out() << _tntChunks[75]; // \n<div class="more withmargin"><a 
  // <& tooltip.hint ...
  { 
#line 341 "schedule.ecpp"
    tnt::QueryParams _tnt_cq18;
#line 341 "schedule.ecpp"
    _tnt_cq18.add( "text", (hint));
#line 341 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "hint"), request, reply, _tnt_cq18);
    // &>
  }
  // <& tooltip.display ...
  { 
#line 341 "schedule.ecpp"
    tnt::QueryParams _tnt_cq19;
#line 341 "schedule.ecpp"
    _tnt_cq19.add( "domId", (id));
#line 341 "schedule.ecpp"
    callComp(tnt::Subcompident(std::string(), "tooltip", "display"), request, reply, _tnt_cq19);
    // &>
  }
  reply.out() << _tntChunks[76]; // ><span class="title">
#line 341 "schedule.ecpp"
  reply.sout() << ( name );
  reply.out() << _tntChunks[77]; // </span><br /><span class="short">
#line 341 "schedule.ecpp"
if (shortDescr.empty()) { 
  reply.out() << _tntChunks[78]; // &nbsp;
#line 341 "schedule.ecpp"
 } 
#line 341 "schedule.ecpp"
  reply.sout() << ( shortDescr );
  reply.out() << _tntChunks[79]; // </span></a></div>\n\n
#line 343 "schedule.ecpp"

#if VDRVERSNUM >= 20505

  reply.out() << _tntChunks[80]; // </td></tr></table>\n
#line 347 "schedule.ecpp"

#endif

  reply.out() << _tntChunks[81]; // </td>\n
  // <%/cpp>
  return DEFAULT;
}

} // namespace
